package java.awt;

import java.io.FilePermission;
import java.net.URISyntaxException;
import java.net.URL;
import sun.awt.DesktopBrowse;
import java.net.MalformedURLException;
import java.net.URI;
import java.io.IOException;
import java.io.File;
import sun.awt.SunToolkit;
import sun.awt.AppContext;
import java.awt.peer.DesktopPeer;

public class Desktop
{
    private DesktopPeer peer;
    
    private Desktop() {
        this.peer = Toolkit.getDefaultToolkit().createDesktopPeer(this);
    }
    
    public static synchronized Desktop getDesktop() {
        if (GraphicsEnvironment.isHeadless()) {
            throw new HeadlessException();
        }
        if (!isDesktopSupported()) {
            throw new UnsupportedOperationException("Desktop API is not supported on the current platform");
        }
        AppContext appContext = AppContext.getAppContext();
        Desktop value = (Desktop)appContext.get(Desktop.class);
        if (value == null) {
            value = new Desktop();
            appContext.put(Desktop.class, value);
            return value;
        }
        return value;
    }
    
    public static boolean isDesktopSupported() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        return defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isDesktopSupported();
    }
    
    public boolean isSupported(Desktop.Action action) {
        return this.peer.isSupported(action);
    }
    
    private static void checkFileValidation(File file) {
        if (!file.exists()) {
            throw new IllegalArgumentException("The file: " + file.getPath() + " doesn't exist.");
        }
        return;
    }
    
    private void checkActionSupport(Desktop.Action action) {
        if (!this.isSupported(action)) {
            throw new UnsupportedOperationException("The " + action.name() + " action is not supported on the current platform!");
        }
        return;
    }
    
    private void checkAWTPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new AWTPermission("showWindowWithoutWarningBanner"));
        }
    }
    
    public void open(File file) throws IOException {
        file = new File(file.getPath());
        this.checkAWTPermission();
        this.checkExec();
        this.checkActionSupport(Desktop.Action.OPEN);
        checkFileValidation(file);
        this.peer.open(file);
    }
    
    public void edit(File file) throws IOException {
        file = new File(file.getPath());
        this.checkAWTPermission();
        this.checkExec();
        this.checkActionSupport(Desktop.Action.EDIT);
        file.canWrite();
        checkFileValidation(file);
        this.peer.edit(file);
    }
    
    public void print(File file) throws IOException {
        file = new File(file.getPath());
        this.checkExec();
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        this.checkActionSupport(Desktop.Action.PRINT);
        checkFileValidation(file);
        this.peer.print(file);
    }
    
    public void browse(URI uri) throws IOException {
        SecurityException ex = null;
        try {
            this.checkAWTPermission();
            this.checkExec();
        }
        catch (SecurityException ex2) {
            ex = ex2;
        }
        this.checkActionSupport(Desktop.Action.BROWSE);
        if (uri == null) {
            throw new NullPointerException();
        }
        if (ex == null) {
            this.peer.browse(uri);
            return;
        }
        URL url;
        try {
            url = uri.toURL();
        }
        catch (MalformedURLException cause) {
            throw new IllegalArgumentException("Unable to convert URI to URL", cause);
        }
        DesktopBrowse instance = DesktopBrowse.getInstance();
        if (instance == null) {
            throw ex;
        }
        instance.browse(url);
        return;
    }
    
    public void mail() throws IOException {
        this.checkAWTPermission();
        this.checkExec();
        this.checkActionSupport(Desktop.Action.MAIL);
        try {
            this.peer.mail(new URI("mailto:?"));
        }
        catch (URISyntaxException ex) {}
    }
    
    public void mail(URI uri) throws IOException {
        this.checkAWTPermission();
        this.checkExec();
        this.checkActionSupport(Desktop.Action.MAIL);
        if (uri == null) {
            throw new NullPointerException();
        }
        if (!"mailto".equalsIgnoreCase(uri.getScheme())) {
            throw new IllegalArgumentException("URI scheme is not \"mailto\"");
        }
        this.peer.mail(uri);
        return;
    }
    
    private void checkExec() throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new FilePermission("<<ALL FILES>>", "execute"));
        }
    }
    
    public enum Action
    {
        OPEN, 
        EDIT, 
        PRINT, 
        MAIL, 
        BROWSE;
    }
}
